<?php
/* Copyright start
  Copyright (C) 2008 - 2022 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\EventSubscriber;

use App\Traits\Trackable;
use App\Entity\Base\BaseEntity;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\ViewEvent;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;

class TrackableSubscriber implements EventSubscriberInterface
{

    protected $containers;
    protected $tokenStorage;
    protected $entityManager;
    protected $requeststack;
    protected $cache;
    protected $dataPersister;

    public function __construct(
        ContainerInterface $containers,
        TokenStorageInterface $tokenStorage,
        EntityManagerInterface $entityManager
    ) {
        $this->containers = $containers;
        $this->tokenStorage = $tokenStorage;
        $this->entityManager = $entityManager;
    }

    public static function getSubscribedEvents()
    {
        return [
            KernelEvents::VIEW => ['handleTrackableFields', 35]
        ];
    }

    /**
     * @param $object
     * @return bool
     */
    private function isTrackable($object, $request = null)
    {
        if ($request && $request->attributes->get('fsr_subrequest_type') === 'import') {
            return false;
        }
        return in_array(Trackable::class, class_uses(get_class($object)));
    }

    public function handleTrackableFields(ViewEvent $event)
    {
        $object = $event->getControllerResult();
        $request = $event->getRequest();
        $request_method = $request->getMethod();
        if (
            !($object instanceof BaseEntity) ||
            in_array($request_method, [Request::METHOD_GET, Request::METHOD_DELETE, Request::METHOD_POST]) ||
            !$this->isTrackable($object, $request)
        ) {
            return;
        }
        $fsr_previous_data = $request->attributes->get('fsr_previous_data');
        list($user, $now) = $this->getTrackableInfo();
        $object->setModifyDate($now);
        if (!is_null($user)) {
            $object->setModifyUser($user);
        }
        if (is_null($fsr_previous_data)) {
            $object->setCreateDate($now);
            $object->setCreateUser($user);
        }
    }

    /**
     * @return array
     */
    private function getTrackableInfo()
    {
        $userToken = $this->tokenStorage->getToken();
        $user = null;
        if (!is_null($userToken)) {
            $user = $userToken->getUser();
        }
        $now = new \DateTime('NOW', new \DateTimeZone('UTC'));
        return array($user, $now);
    }
}
